require "Apollo"

local UIFixes = {}

UIFixes.Settings = {
	sortcx = true,
	stickywhisper = true,
}

function UIFixes:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    -- initialize variables here

    return o
end

function UIFixes:Init()
    local bHasConfigureFunction = true
    local strConfigureButtonText = "AdeleaUIFixes"
    local tDependencies = nil
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function UIFixes:OnConfigure()
	if self.xmlCfg and self.xmlCfg:IsLoaded() then
		self.wndCfg:Show(true, true)
	end
end

function UIFixes:SortedCX()
	UIFixes.MPC = Apollo.GetAddon("MarketplaceCommodity")
	if UIFixes.MPC then
		function UIFixes.MPC.InitializeSell(self)
			local unitPlayer = GameLib.GetPlayerUnit()
			local tBothItemTables = {}
			local tItemsAdded = {}
			for key, tCurrData in pairs(unitPlayer:GetInventoryItems()) do
				-- table.insert(tBothItemTables, tCurrData.itemInBag:GetItemId(), tCurrData.itemInBag)
				local strName = tCurrData.itemInBag:GetName()
				if not tItemsAdded[strName] then
					tItemsAdded[strName] = true
					table.insert(tBothItemTables, tCurrData.itemInBag)
				end
			end
			for key, tSatchelItemCategory in pairs(unitPlayer:GetSupplySatchelItems(1)) do
				for key2, tCurrData in pairs(tSatchelItemCategory) do
					-- table.insert(tBothItemTables, tCurrData.itemMaterial:GetItemId(), tCurrData.itemMaterial)
					local strName = tCurrData.itemMaterial:GetName()
					if not tItemsAdded[strName] then
						tItemsAdded[strName] = true
						table.insert(tBothItemTables, tCurrData.itemMaterial)
					end
				end
			end
			table.sort(tBothItemTables, function(a,b) return a:GetName() < b:GetName() end)
			for key, tCurrItem in pairs(tBothItemTables) do
				if tCurrItem and tCurrItem:IsCommodity() then
					local bSellNow = self.wndMain:FindChild("HeaderSellNowBtn"):IsChecked()
					local strWindow = bSellNow and "SimpleListItem" or "AdvancedListItem"
					local strButtonText = bSellNow and Apollo.GetString("MarketplaceCommodity_SellNow") or Apollo.GetString("MarketplaceCommodity_CreateSellOrder")
					self:BuildListItem(tCurrItem, strWindow, strButtonText)

					-- TODO: Count the number of request and load spinner until they all come back
					MarketplaceLib.RequestCommodityInfo(tCurrItem:GetItemId()) -- Leads to OnCommodityInfoResults
				end
			end			
		end	
	end
end

function UIFixes:StickyWhisper()
	local _chataddons = {
		"ChatLog",
		"ChatFixed",
		"ImprovedChatLog",
		"BetterChatLog",
	}
		
	local _chatprefix = {
		["/w"] = true,
		["/t"] = true,
		["/aw"] = true,
	}
	
	local _chatchannels = {
		[ChatSystemLib.ChatChannel_AccountWhisper] = "/aw",
		[ChatSystemLib.ChatChannel_Whisper] = "/w"
	}

	for _,v in pairs(_chataddons) do
		UIFixes.CL = Apollo.GetAddon(v)
		if UIFixes.CL then
			UIFixes.OnChatInputReturn = UIFixes.CL.OnChatInputReturn
			function UIFixes.CL.OnChatInputReturn(s,h,c,t)
				local r = {}
				for token in string.gmatch(t, "[^%s]+") do
					table.insert(r, token)
				end

				local tChatData = c:GetParent():GetData()
				local chanCurrent = tChatData.channelCurrent:GetType()
				
				if _chatprefix[r[1]] and r[2] then
					UIFixes.tChatPrefix = r[1]
					UIFixes.tLastWhisperer = r[2]
				end
				if _chatchannels[chanCurrent] and UIFixes.tLastWhisperer then
					if string.sub(t,1,1) ~= "/" then
						t = UIFixes.tChatPrefix .. " " .. UIFixes.tLastWhisperer .. " " .. t
					end
				end
				UIFixes.OnChatInputReturn(s,h,c,t)
			end
			break
		end
	end
end

function UIFixes:SetupFixes()
	if UIFixes.Settings.sortcx then
		self:SortedCX()
	end
	if UIFixes.Settings.stickywhisper then
		self:StickyWhisper()
	end
end

function UIFixes:OnLoad()
	self.xmlCfg = XmlDoc.CreateFromFile("UIFixes_Config.xml")
	self.xmlCfg:RegisterCallback("OnCfgLoaded", self)
end

function UIFixes:OnButtonChange()
	UIFixes.Settings.sortcx = UIFixes.bSortCX:IsChecked()
	UIFixes.Settings.stickywhisper = UIFixes.bSticky:IsChecked()
end

function UIFixes:OnCloseButton()
	self.wndCfg:Show(false, true)
end

function UIFixes:OnCfgLoaded()
	if self.xmlCfg and self.xmlCfg:IsLoaded() then
		self.wndCfg = Apollo.LoadForm(self.xmlCfg, "UIFixesConfig", nil, self)
		
		UIFixes.bSortCX = self.wndCfg:FindChild("bSortCX")
		UIFixes.bSticky = self.wndCfg:FindChild("bSticky")
		UIFixes.bBlockArena = self.wndCfg:FindChild("bBlockArena")
		
		UIFixes.bSortCX:SetCheck(UIFixes.Settings.sortcx)
		UIFixes.bSticky:SetCheck(UIFixes.Settings.stickywhisper)
		
		self.wndCfg:Show(false, true)
	end
	self:SetupFixes()
end

function UIFixes:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		return UIFixes.Settings
	end
end

function UIFixes:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		for k,v in pairs(tSavedData) do
			UIFixes.Settings[k] = v
		end
	end
end

local UIFixesInst = UIFixes:new()
UIFixesInst:Init()
